package cs4hs11.rsalibrary;
public class RSAMath {
	/** 
	 * Empty constructor.  This class contains no state; as a result,
	 * this class could contain no more than static utility functions.
	 * It is instead implemented as a standard class with no state,
	 * and a suite of self-contained functions.  Therefore, instantiate
	 * an object of this class RSAMath x = new RSAMath(); and call its functions
	 * like x.GCD(5, 8);
	 */
	public RSAMath() {
		
	}
	
	/**
	 * Computes the GCD of two numbers a and b
	 * @param a One of the two numbers to compute the GCD on
	 * @param b One of the two numbers to compute the GCD on
	 * @return The GCD of the input parameters a and b, as an integer
	 */
	public long GCD(long a, long b)
	{
	   if (b==0) return a;
	   return GCD(b,a%b);
	}
	
	/**
	 * Computes Math.mod(Math.pow(a, b), c), for large values of a, b, and c
	 * @param a The base value a in a^b % c
	 * @param b The exponent value b in a^b % c
	 * @param c The modulus value c in a^b % c
	 * @return The mod power a^b mod c
	 */
	public long modulo(long a,long b,long c) {
		long x=1;
		long y=a;
		while(b > 0){
			if(b%2 == 1){
				x=(x*y)%c;
			}
			y = (y*y)%c; // squaring the base
			b /= 2;
		}
		return (long) x%c;
	}
	
	/**
	 * Compute Euler's Totient.  From Wikipedia: The totient of a positive integer n
	 * is defined to be the number of positive integers less than or equal to n that
	 * are coprime to n (in other words, they have no common positive factors other than 1).
	 * In terms of the GCD, the Totient is the number of numbers for each i such that 1 <= i <= n, 
	 * with gcd(i, n) = 1.  In particular, tot(1) = 1, since 1 is coprime to itself (1 is the only
	 * natural number with this property).  tot(9) = 6, since the nubmers 1, 2, 4, 5, 7, and 8 are 
	 * coprime to 9.  This is important in the RSA cryptosystem, because Euler's theorem goes on
	 * to state that a^(tot(n)) = 1 (mod n) for all a coprime to n. 
	 * @param n The number to compute the Totient on
	 * @return The Euler Totient value tot(n)
	 */
	public long totient(long n) { 
	
		long count = 0; 
		
		for(long i = 1; i < n; i++) 
		if(GCD(n,i) == 1) 
		count++; 
		
		return count; 
	
	} 
	
	/**
	 * Using a java.util.Random number generator, pick a random integer
	 * that is coprime to the given input parameter x.
	 * @param x An integer for which a coprime number is desired.
	 * @return Any coprime integer to the input x.
	 */
	public long coprime(long x) {
		long iTheCoPrime = x + 1;
		
		while(iTheCoPrime >= x) {
			java.util.ArrayList<Long> coprimes = new java.util.ArrayList<Long>();
			
			for(long i = 2; i < x; i++) {
				if(GCD(i, x) == 1) {
					coprimes.add(new Long(i));
				}
			}
			
			java.util.Random rand = new java.util.Random();
			int rand_index = rand.nextInt(coprimes.size());
			Long thecoprime = (Long) coprimes.get(rand_index);
			iTheCoPrime = thecoprime.longValue();
		}
		
		return iTheCoPrime;
	}
	
	/**
	 * Compute the modular inverse base^-1 % m.  Note this is not the arithmetic inverse (1/base) mod m.
	 * @param base The base value for which the modular inverse mod m is desired. 
	 * @param m The modulo parameter m for which the modular inverse of base mod m is desired.
	 * @return The modulo inverse of base mod m; in other words, (base^-1) mod m.  
	 */
	public long mod_inverse(long base, long m) {
		long tot = totient(m) - 1;
		long inv = modulo(base, tot, m);
		return inv;
	}
	
	/**
	 * Given an integer representing an ASCII character value, encrypt it 
	 * via the RSA crypto algorithm.  Or, given an integer representing an 
	 * encrypted ASCII value, decrypt it via the RSA crypto algorithm.  
	 * Encryption is performed by taking the public key (which could have been
	 * either RSA key computed during key creation) modulo parameter C, and
	 * the ASCII value X (in msg_or_cipher), and computing (X^key) mod c.
	 * Decryption is done in the same way, but Y (msg_or_cipher) is passed
	 * instead as the encrypted integer value, and the key passed is the
	 * private key.  This process will work regardless of which key is used,
	 * so the private key can be used to encrypt values that the public key
	 * will decrypt (since they are modular inverses of one another), thanks to
	 * Euler's Theorem.  This is useful for computing message signatures. 
	 * @param msg_or_cipher An integer containing the ASCII value to be encrypted or the encrypted integer value to be decrypted back to an ASCII value.
	 * @param key The public key or private key value as an integer, as appropriate.
	 * @param c The modulo parameter for performing encryption or decryption.  This value is part of the public and private key (it is shared between the two!). 
	 * @return The encrypted (or decrypted) value according to the key and modulo C
	 */
	public long endecrypt(long msg_or_cipher, long key, long c) {
		return modulo(msg_or_cipher, key, c);
	}
	
	/**
	 * Given a value n, get the n'th prime number
	 * @param n The value n that represents the n'th prime number you want (1st prime, 2nd prime, etc.)
	 * @return The n'th prime number.
	 */
	public long getNthPrime(long n) {
		long k = 0;
		long p = 2;
		
		while(k < n) {
			p++;
			if(isprime(p)) k++;
		}
		
		return p;
	}
	
	/**
	 * Determine if a number is prime
	 * @param p The number to determine primality
	 * @return True if the number is prime, false otherwise.
	 */
	public boolean isprime(long p) {
		for(long i = 2; i < p; i++) { // This could really be Math.sqrt(p) instead of p
			if(p % i == 0) {
				return false;
			}
		}
		
		return true;
	}	
}